/*
 * dest.c - test DES algorithm
 */

#include <stdio.h>
#include <string.h>
#if !unix
#include <stdlib.h>
#endif /* !unix */

#include "des_crypt.h"


typedef char bool;
#define	YES	1
#define	NO	0
typedef unsigned char uchar;
typedef unsigned long ulong;



static char *Progname = "dest";




void usage()
{
	fprintf(stderr, "Usage: %s [-d] hex-key hex-input hex-output\n", Progname);
}


/*
 * convert hexadecimal string 'src' into binary vector dst[size]
 */
void get_hex_string(dst, src, size)
	char *dst;
	char *src;
	int size;
{
	for (; size; size--) {
		char buf[3];
		char *p;

		if (*src) {
			buf[0] = *src++;
			if (*src == '\0') {
				fprintf(stderr, "Odd hex string\n");
				exit(1);
			}
			buf[1] = *src++;
			buf[2] = '\0';
			*dst++ = strtol(buf, &p, 16);
			if (*p != '\0') {
				fprintf(stderr, "Illegal hex digit: %c\n", *p);
				exit(1);
			}
		} else
			*dst++ = 0;
	}
}



int main(argc, argv)
	int argc;
	char **argv;
{
	unsigned dflag = DES_ENCRYPT | DES_HW;
	int pflag = 0;
	uchar key[8], block[8], output[8];

	while (++argv, --argc > 0 && **argv == '-' && argv[0][1]) {
		switch (argv[0][1]) {
		case 'd':
			dflag = DES_DECRYPT | DES_HW;
			break;
		case 'p':
			pflag = 1;
			break;
		default:
			fprintf(stderr, "Bad option: %s\n", *argv);
			usage();
			exit(1);
		}
	}
	if (argc != 3) {
		usage();
		exit(1);
	}
	memset(key, 0, 8);
	memset(block, 0, 8);
	memset(output, 0, 8);
	get_hex_string(key, *argv++, 8);
	get_hex_string(block, *argv++, 8);
	get_hex_string(output, *argv, 8);
	if (!pflag) des_setparity(key);
	printf("   key = %02X:%02X:%02X:%02X:%02X:%02X:%02X:%02X\n",
		key[0], key[1], key[2], key[3],
		key[4], key[5], key[6], key[7]);
	printf(" input = %02X:%02X:%02X:%02X:%02X:%02X:%02X:%02X\n",
		block[0], block[1], block[2], block[3],
		block[4], block[5], block[6], block[7]);
	ecb_crypt(key, block, 8, dflag);
	printf("output = %02X:%02X:%02X:%02X:%02X:%02X:%02X:%02X...",
		block[0], block[1], block[2], block[3],
		block[4], block[5], block[6], block[7]);
	if (memcmp(output, block, 8) == 0) {
		printf("OK!\n");
		exit(0);
	} else {
		printf("NG!\n");
		printf("expect = %02X:%02X:%02X:%02X:%02X:%02X:%02X:%02X\n",
		       output[0], output[1], output[2], output[3],
		       output[4], output[5], output[6], output[7]);
		exit(1);
	}
}

